{
    db => {
        servers => {
            server1 => {
                host  => 'host1',
                user  => 'user1',
            },
            server2 => {
                host  => 'host2',
                user  => 'user2',
            },
            list  => [qw(server1 server2)],
        }
    }
}
